use std::{collections::HashMap, fs};

use log::error;

// ENUMS

/// Specifies a moderation action, whether block or silence
#[derive(Clone, Copy, Debug, Eq, PartialEq)]
pub enum ModAction {
    Block,
    Silence,
}

// STRUCTS

/// Indicates weight of moderation action on a host
#[derive(Debug, Default, Eq, PartialEq)]
pub struct ModActionTrust {
    pub block: u16,
    pub silence: u16,
}

impl ModActionTrust {
    pub fn add_action(&mut self, action: ModAction, trust: u16) -> &mut Self {
        match action {
            ModAction::Block => self.block += trust,
            ModAction::Silence => self.silence += trust,
        }
        self
    }
}

impl From<(u16, u16)> for ModActionTrust {
    fn from(value: (u16, u16)) -> Self {
        Self {
            block: value.0,
            silence: value.1,
        }
    }
}

/// Contains a mapping of hosts to moderation actions as well as a trust value,
/// which is used to weight moderation actions when building a mod map
#[derive(Debug, PartialEq, Eq)]
pub struct ModSource {
    pub actions: HashMap<String, ModAction>,
    pub trust: u16,
}

impl Default for ModSource {
    fn default() -> Self {
        Self {
            actions: HashMap::new(),
            trust: 100,
        }
    }
}

impl From<HashMap<String, ModAction>> for ModSource {
    fn from(map: HashMap<String, ModAction>) -> Self {
        Self {
            actions: map,
            trust: 100,
        }
    }
}

impl ModSource {
    fn add_action(&mut self, host: &str, action: ModAction) -> &mut Self {
        self.actions.insert(host.to_string(), action);
        self
    }

    pub fn build(map: HashMap<String, ModAction>, trust: u16) -> Self {
        let mut src = Self::from(map);
        src.trust = trust;
        src
    }

    pub fn import_file(&mut self, path: &str, action: ModAction) -> &mut Self {
        let contents = fs::read_to_string(path).unwrap();
        for host in contents.lines().filter(|line| !line.is_empty()) {
            self.add_action(host, action);
        }

        self
    }
}

/// A map of hosts (as strings) to weighted mod actions
#[derive(Debug, Default, Eq, PartialEq)]
pub struct ModMap(pub HashMap<String, ModActionTrust>);

impl ModMap {
    pub fn add_source(&mut self, src: ModSource) -> &mut Self {
        let items = src.actions.into_iter();

        for (host, action) in items {
            let entry = self.0.entry(host).or_default();
            entry.add_action(action, src.trust);
        }

        self
    }

    pub fn export_file(self, block_path: &str, mute_path: &str, heat: (u16, u16)) -> std::io::Result<()> {
        if self.0.is_empty() {
            error!("Nothing to export!");
            return Ok(())
        }

        let (block_thresh, mute_thresh) = heat;
        let mut block_output: String = String::default();
        let mut mute_output: String = String::default();

        for item in self.0.into_iter() {
            let (block_trust, mute_trust) = (item.1.block, item.1.silence);

            if block_trust >= block_thresh {
                block_output.push_str(&(item.0.clone() + "\n"));
            }

            if mute_trust >= mute_thresh {
                mute_output.push_str(&(item.0.clone() + "\n"));
            }
        }

        fs::write(block_path, block_output)?;
        fs::write(mute_path, mute_output)?;

        Ok(())
    }
}
